package org.bitbucket.bradleysmithllc.etlunit.feature.database.db;

import org.bitbucket.bradleysmithllc.etlunit.util.StringUtils;

import java.util.*;

public class CatalogImpl implements Catalog
{
	private final String name;
	private final Database database;

	private final List<Schema> schemaList = new ArrayList<Schema>();
	private final Map<String, Schema> schemaMap = new HashMap<String, Schema>();

	public CatalogImpl(Database database, String name) {
		this.name = name;
		this.database = database;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public Database getDatabase() {
		return database;
	}

	@Override
	public List<Schema> getSchemas() {
		return Collections.unmodifiableList(schemaList);
	}

	@Override
	public Map<String, Schema> getSchemaMap() {
		return schemaMap;
	}

	@Override
	public Schema getSchema(String name) {
		String safeNullableUppercase = StringUtils.getSafeNullableUppercase(name);
		Schema s = schemaMap.get(safeNullableUppercase);

		if (s == null)
		{
			return addSchema(new SchemaImpl(this, safeNullableUppercase));
		}

		return s;
	}

	public Schema addSchema(Schema schema)
	{
		String safeNullableUppercase = StringUtils.getSafeNullableUppercase(schema.getName());
		if (schemaMap.containsKey(safeNullableUppercase))
		{
			throw new IllegalArgumentException("Schema name [" + safeNullableUppercase + "] already added to catalog");
		}

		schemaList.add(schema);
		schemaMap.put(safeNullableUppercase, schema);

		return schema;
	}

	public String toString()
	{
		return database.toString() + "." + (name == null ? "[DEFAULT]" : name.toString());
	}
}